from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING

from haco.base import Base
from haco.topics import AnnounceTopicState, AnnounceTopicCommand

if TYPE_CHECKING:
    from haco.control import Control


@dataclass(kw_only=True)
class Capability(Base):
    name: str

    state: AnnounceTopicState | None = field(default_factory=AnnounceTopicState)
    command: AnnounceTopicCommand | None = field(default_factory=AnnounceTopicCommand)
    subscriptions: dict | None = field(default=None, metadata=dict(exclude=True), init=False)

    parent: Control | None = None
    announce = None

    def set_parent(self, control):
        self.parent = control
        for topic in (t for t in (self.state, self.command) if t):
            topic.set_parent(self)

        self.subscriptions = self.get_subscriptions()

        self.announce = self.get_announce()

    def get_announce(self) -> dict:
        data = {}
        for topic in (t for t in (self.state, self.command) if t):
            data |= topic.announce
        return data

    @property
    def control(self):
        return self.parent

    def get_subscriptions(self) -> dict:
        data = {}
        for topic in (t for t in (self.state, self.command) if t):
            data |= topic.subscriptions
        return data



from __future__ import annotations

from dataclasses import dataclass, field
from functools import cached_property
from typing import ClassVar, TYPE_CHECKING

from haco.obs import logger

if TYPE_CHECKING:
    from capabilities import Capability


@dataclass(kw_only=True)
class AnnounceTopic:
    IO: ClassVar[str | None] = None
    key: str = '{capability}_{io}_topic'
    value: str = '{path}/{capability}/{io}'

    parent: Capability | None = None
    announce: dict | None = field(default=None, metadata=dict(exclude=True))
    subscriptions: dict | None = field(default=None, metadata=dict(exclude=True), init=False)

    def set_parent(self, capability: Capability):
        self.parent = capability
        self.announce = self.get_announce()
        self.subscriptions = self.get_subscriptions()

    def fill(self, mask):
        return mask.format(**self.fills)

    @property
    def fills(self):
        return dict(
            path=str(self.capability.control.topic),
            capability=self.capability.name,
            io=self.IO
        )

    @cached_property
    def topic_key(self):
        return self.fill(self.key)

    @cached_property
    def topic(self):
        return self.fill(self.value)

    def get_announce(self):
        return {self.topic_key: self.topic}

    @property
    def callback_name(self):
        return f'{self.capability.name}_{self.IO}'

    def get_subscriptions(self):
        return {}

    @cached_property
    def capability(self):
        return self.parent


@dataclass(kw_only=True)
class AnnounceTopicState(AnnounceTopic):
    IO: ClassVar[str] = 'state'

    def get_subscriptions(self):
        return {}

    async def wrap_back(self, value):
        method = getattr(self.capability.control, self.callback_name, None)

        try:
            value_raw = method(value)
        except NotImplementedError:
            logger.warning(f'No method implemented for {self.topic}')
            return

        await self.capability.control.device.client.publish(self.topic, value_raw)
        return value_raw

@dataclass(kw_only=True)
class AnnounceTopicCommand(AnnounceTopic):
    IO: ClassVar[str] = 'command'

    @property
    def state(self):
        return self.capability.state

    def wrap_back(self, message):
        method = getattr(self.capability.control, self.callback_name, None)

        try:
            value = method(message.payload.decode('utf-8'))
        except NotImplementedError:
            logger.warning(f'No method implemented for {self.topic}')
            return

        return value

    def get_subscriptions(self):
        return {self.topic: self}



import asyncio

from haco.client import ClientHaco
from haco.climate import Climate
from haco.constants import MQTT_HOST
from haco.device import Device


async def main():
    climate_a = Climate(name="Dev Climate A", fan_modes=['a', 'b', 'c'])
    climate_b = Climate(name="Dev Climate B", fan_modes=['a', 'b', 'c'])
    device = Device(name="Dev Device", controls=[climate_a, climate_b])

    client = ClientHaco(hostname=MQTT_HOST, device=device)

    await client.start()




if __name__ == "__main__":
    asyncio.run(main())



from dataclasses import dataclass
from typing import List, Optional, Literal

from capabilities import Capability
from control import Control


@dataclass(kw_only=True)
class Climate(Control):
    platform: Literal["climate"] = "climate"
    temperature_unit: Literal["C", "F"] = "C"
    temperature_min: float = 0
    temperature_max: float = 40
    humidity_min: float = 0
    humidity_max: float = 100
    precision: int = 1
    modes: Optional[List[str]] = None
    preset_modes: Optional[List[str]] = None
    fan_modes: Optional[List[str]] = None
    swing_modes: Optional[List[str]] = None


    def temperature_command(self, value):
        raise NotImplementedError()

    def mode_command(self, value):
        raise NotImplementedError()

    def fan_mode_command(self, value):
        raise NotImplementedError()

    def fan_mode_state(self, value):
        raise NotImplementedError()

    def power_command(self, value):
        raise NotImplementedError()

    def power_state(self, value):
        raise NotImplementedError()

    @classmethod
    def get_capabilities(cls):
        return [
            Capability(name="current_temperature", command=None),
            Capability(name="temperature"),
            Capability(name="current_humidity"),
            Capability(name="target_humidity"),
            Capability(name="mode"),
            Capability(name="fan_mode"),
            Capability(name="power"),
        ]



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING

from haco.base import Base
from haco.capabilities import Capability
from haco.utils import sanitize_name

if TYPE_CHECKING:
    from haco.device import Device


@dataclass(kw_only=True)
class Control(Base):
    name: str
    platform: str

    device: Device = field(default=None, init=False)
    capabilities: None | list[Capability] = field(default=None, metadata=dict(exclude=True))
    unique_id: str | None = field(default=None, init=False)
    availability_topic: str | None = field(default=None, init=False)

    announce: dict | None = field(default=None, metadata=dict(exclude=True))
    subscriptions: dict | None = field(default=None, metadata=dict(exclude=True), init=False)

    def __post_init__(self):
        self.capabilities = self.get_capabilities()

    def set_parent(self, device):
        self.device = device

        for cap in self.capabilities:
            cap.set_parent(self)

        self.unique_id = self.get_unique_id()
        self.availability_topic = self.get_availability_topic()
        self.subscriptions = self.get_subscriptions()
        self.announce = self.get_announce()

    @property
    def parent(self):
        return self.device

    def get_announce(self):
        data = self.model_dump()
        for cap in self.capabilities:
            data |= cap.announce

        data = {self.announce_topic: data}
        return data

    @classmethod
    def get_capabilities(cls):
        return [
            Capability(name='default')
        ]

    def get_unique_id(self) -> str:
        return f"{self.device.name_san}-{self.name_san}"

    @property
    def name_san(self) -> str:
        return sanitize_name(self.name)

    @property
    def topic(self):
        return self.device.topic / self.name_san

    @property
    def announce_topic(self):
        return f"homeassistant/{self.platform}/{self.unique_id}/config"

    def get_availability_topic(self) -> str:
        return self.device.client.will.topic

    def get_subscriptions(self) -> dict:

        data = {}

        for capability in self.capabilities:
            data |= capability.subscriptions

        return data

    async def initialise(self):
        ...


if __name__ == "__main__":
    control = Control(name="test", platform="test")
    control



import string
from dataclasses import dataclass

CHARS_ALLOWED = string.ascii_lowercase + string.digits
SEPS = '_- /'
BRANCH = 'release'

CAPABILITY_DEFAULT = 'default'
RECONNECT_DELAY_SEC = 5
ON = 'ON'
OFF = 'OFF'


def sanitize_name(s, sep='-'):
    chars = []

    for c in s.lower():
        if c in SEPS:
            chars.append(sep)
        elif c in CHARS_ALLOWED:
            chars.append(c)

    sanitized = ''.join(chars)

    if not sanitized:
        raise ValueError("Sanitized string is empty")

    return sanitized


@dataclass
class Metadata:
    exclude: bool = False
